@@ -1,3 +1,201 @@
+# The ubUnder(), ubOver(), and ubSMOTE() functions were taken from the unbalanced package and modified.
+# Unbalanced packages have not been upgraded since 2015. 
+# And it was removed from CRAN because it did not meet CRAN's criteria. 
+# So I take the core logic and use it.
+ubUnder <- function (X, Y, perc = 50) {
+  stopifnot(all(unique(Y) %in% c(0, 1)))
+  
+  N <- length(Y)
+  
+  i_pos <- which(Y == 1)
+  n_pos <- length(i_pos)
+  
+  i_neg <- which(Y == 0)
+  n_neg <- length(i_neg)
+  
+  if (n_pos >= n_neg) {
+    stop("less 0s instances than 1s, the minority class has to be class 1")
+  }
+  
+  stopifnot(perc >= (n_pos/N * 100), perc <= 50)
+  n_neg_sub <- floor(n_pos * (100 - perc)/perc)
+  
+  w <- rep(1/n_neg, n_neg)
+  
+  if (n_neg_sub <= n_neg) {
+    i_neg_sub <- sample(i_neg, n_neg_sub, prob = w)
+  } else {
+    stop("subset of majoirty instances bigger than orginal set of majoirty instances")
+  }
+  
+  idx <- c(i_neg_sub, i_pos)
+  idx <- sort(idx)
+  
+  if (is.vector(X) != TRUE) {
+    X <- X[idx, ]
+  } else {
+    X <- X[idx]
+  }
+  
+  Y <- Y[idx]
+  
+  return(list(X = X, Y = Y))
+}
+
+
+ubOver <- function (X, Y, k = 0, verbose = TRUE) {
+  stopifnot(k >= 0, class(verbose) == "logical", all(unique(Y) %in% c(0, 1)))
+  
+  i_pos <- which(Y == 1)
+  n_pos <- length(i_pos)
+  i_neg <- which(Y == 0)
+  n_neg <- length(i_neg)
+  
+  max_k <- floor(n_neg/n_pos)
+  
+  if (k == 0) {
+    i_pos_over <- sample(i_pos, n_neg, replace = TRUE)
+  }
+  
+  if (k > 0) {
+    n_pos_over <- n_pos * k
+    
+    if (n_pos_over > n_neg) {
+      if (verbose) 
+        cat("Max number of times allowed to replicate minority class is", 
+            max_k, "\n taking as many samples as the majority class \n")
+      n_pos_over <- n_neg
+    }
+    
+    i_pos_over <- sample(i_pos, n_pos_over, replace = TRUE)
+  }
+  
+  idx = c(i_neg, i_pos_over)
+  idx <- sort(idx)
+  
+  if (is.vector(X) != TRUE) {
+    X = X[idx, ]
+  } else {
+    X = X[idx]
+  }
+  
+  Y = Y[idx]
+  
+  return(list(X = X, Y = Y))
+}
+
+#' @importFrom stats runif
+ubSmoteExs <- function (data, N = 200, k = 5) {
+  nomatr <- c()
+  
+  n_row <- NROW(data)
+  n_col <- NCOL(data)
+  
+  mat <- matrix(nrow = n_row, ncol = n_col - 1)
+  
+  for (col in seq.int(NCOL(mat))) {
+    obs <- data[, col]
+    cl <- class(obs)
+    
+    if (cl %in% c("Date", "POSIXct", "POSIXt")) 
+      stop("cannot SMOTE variables of class Date, POSIXct or POSIXt")
+    
+    if (cl %in% c("factor", "character")) {
+      mat[, col] <- as.integer(obs)
+      nomatr <- c(nomatr, col)
+    } else {
+      mat[, col] <- obs
+    }
+  }
+  
+  if (N < 100) {
+    idx <- sample(1:n_row, as.integer((N/100) * n_row))
+    mat <- mat[idx, ]
+    
+    N <- 100
+  }
+  
+  p <- NCOL(mat)
+  nmat <- NROW(mat)
+  
+  ranges <- apply(mat, 2, max) - apply(mat, 2, min)
+  nexs <- as.integer(N/100)
+  
+  new <- matrix(nrow = nexs * nmat, ncol = p)
+  
+  for (i in 1:nmat) {
+    xd <- scale(mat, mat[i, ], ranges)
+    
+    for (a in nomatr) {
+      xd[, a] <- xd[, a] == 0
+    } 
+    
+    dd <- drop(xd^2 %*% rep(1, ncol(xd)))
+    kNNs <- order(dd)[2:(k + 1)]
+    
+    for (n in 1:nexs) {
+      neig <- sample(1:k, 1)
+      ex <- vector(length = ncol(mat))
+      difs <- mat[kNNs[neig], ] - mat[i, ]
+      new[(i - 1) * nexs + n, ] <- mat[i, ] + stats::runif(1) * difs
+      
+      for (a in nomatr) {
+        new[(i - 1) * nexs + n, a] <- c(mat[kNNs[neig], a], mat[i, a])[1 + round(runif(1), 0)]
+      } 
+    }
+  }
+  
+  newCases <- data.frame(new)
+  
+  for (a in nomatr) {
+    newCases[, a] <- factor(newCases[, a], 
+                            levels = 1:nlevels(data[, a]), 
+                            labels = levels(data[, a]))
+  } 
+  
+  newCases[, "Y"] <- factor(rep(data[1, "Y"], nrow(newCases)), 
+                            levels = levels(data[, "Y"]))
+  colnames(newCases) <- colnames(data)
+  
+  newCases
+}
+
+
+#' @importFrom dplyr bind_rows
+ubSMOTE <- function (X, Y, perc.over = 200, k = 5, perc.under = 200) {
+  if (!is.factor(Y)) 
+    stop("Y has to be a factor")
+  
+  if (is.vector(X)) 
+    stop("X cannot be a vector")
+  
+  data <- cbind(X, Y)
+  idx_pos <- which(Y == 1)
+  
+  newExs <- ubSmoteExs(data[idx_pos, ], perc.over, k)
+  
+  is_narow <- apply(newExs, 1, function(x) any(is.na(x)))
+  
+  if (any(is_narow)) {
+    newExs <- newExs[!is_narow, ]
+    colnames(newExs) <- colnames(data)
+    cat("WARNING: NAs generated by SMOmatE removed \n")
+  }
+  
+  selMaj <- sample((1:NROW(data))[-idx_pos], 
+                   as.integer((perc.under / 100) * nrow(newExs)), 
+                   replace = TRUE)
+  
+  newdataset <- dplyr::bind_rows(data[selMaj, ], data[idx_pos, ], newExs)
+  newdataset <- newdataset[sample(1:NROW(newdataset)), ]
+  
+  X <- newdataset[, -ncol(newdataset)]
+  Y <- newdataset[, ncol(newdataset)]
+  
+  return(list(X = X, Y = Y))
+}
+
+
 #' Extract the data to fit the model
 #'
 #' @description To solve the imbalanced class, perform sampling in the train set of split_df.
@@ -94,7 +292,6 @@
 #' @importFrom tidyselect vars_select
 #' @importFrom rlang quos
 #' @importFrom tibble as_tibble
-#' @importFrom unbalanced ubUnder ubOver ubSMOTE ubOSS
 #' @export
 #'
 sampling_target <- function(.data, method = c("ubUnder", "ubOver", "ubSMOTE"),
@@ -155,20 +352,17 @@ sampling_target <- function(.data, method = c("ubUnder", "ubOver", "ubSMOTE"),
   set.seed(seed)
 
   samples <- switch (method,
-    ubUnder = unbalanced::ubUnder(X, Y, perc = perc, method = "percPos", w = NULL),
-    ubOver  = unbalanced::ubOver(X, Y, k = k),
-    ubOSS  = unbalanced::ubOSS(X, Y),
-    ubSMOTE = unbalanced::ubSMOTE(data.frame(X), Y, perc.over = perc.over, k = k,
-      perc.under = perc.under)
+    ubUnder = ubUnder(X, Y, perc = perc),
+    ubOver  = ubOver(X, Y, k = k),
+    ubSMOTE = ubSMOTE(data.frame(X), Y, perc.over = perc.over, k = k,
+                      perc.under = perc.under)
   )
 
   if (method == "ubUnder") {
     result <- .data %>%
       filter(split_flag == "train") %>%
       ungroup() %>%
-      dplyr::select(-split_flag) %>%
-      .[-c(samples$id.rm), ]
-
+      dplyr::select(-split_flag)
   } else if(method %in% c("ubOver", "ubSMOTE")) {
     Y <- factor(ifelse(samples$Y == 1, minority, majority))
 
@@ -196,10 +390,6 @@ sampling_target <- function(.data, method = c("ubUnder", "ubOver", "ubSMOTE"),
       }
     }
 
-    #result <- samples$X %>%
-    #  mutate_at(attr(.data, "target"), function(x) Y) %>%
-    #  as.tbl()
-
     smpl <- cbind(samples$X, Y)
     names(smpl)[NCOL(smpl)] <- attr(.data, "target")
 
